@@ -38,6 +38,7 @@ G_DEFINE_TYPE (EEwsFolder, e_ews_folder, G_TYPE_OBJECT)
 struct _EEwsFolderPrivate {
 	GError *error;
 	gchar *name;
+	gchar *escaped_name;
 	EwsFolderId *fid;
 	EwsFolderId *parent_fid;
 	EEwsFolderType folder_type;
@@ -48,8 +49,6 @@ struct _EEwsFolderPrivate {
 	gboolean foreign;
 };
 
-static GObjectClass *parent_class = NULL;
-
 static void
 e_ews_folder_dispose (GObject *object)
 {
@@ -57,8 +56,7 @@ e_ews_folder_dispose (GObject *object)
 
 	g_return_if_fail (E_IS_EWS_FOLDER (folder));
 
-	if (parent_class->dispose)
-		(* parent_class->dispose) (object);
+	G_OBJECT_CLASS (e_ews_folder_parent_class)->dispose (object);
 }
 
 static void
@@ -78,6 +76,9 @@ e_ews_folder_finalize (GObject *object)
 		priv->name = NULL;
 	}
 
+	g_free (priv->escaped_name);
+	priv->escaped_name = NULL;
+
 	if (priv->fid) {
 		g_free (priv->fid->id);
 		g_free (priv->fid->change_key);
@@ -92,11 +93,7 @@ e_ews_folder_finalize (GObject *object)
 		priv->parent_fid = NULL;
 	}
 
-	g_free (priv);
-	folder->priv = NULL;
-
-	if (parent_class->finalize)
-		(* parent_class->finalize) (object);
+	G_OBJECT_CLASS (e_ews_folder_parent_class)->finalize (object);
 }
 
 static void
@@ -104,7 +101,7 @@ e_ews_folder_class_init (EEwsFolderClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
-	parent_class = g_type_class_peek_parent (klass);
+	g_type_class_add_private (klass, sizeof (EEwsFolderPrivate));
 
 	object_class->dispose = e_ews_folder_dispose;
 	object_class->finalize = e_ews_folder_finalize;
@@ -113,15 +110,10 @@ e_ews_folder_class_init (EEwsFolderClass *klass)
 static void
 e_ews_folder_init (EEwsFolder *folder)
 {
-	EEwsFolderPrivate *priv;
-
-	/* allocate internal structure */
-	priv = g_new0 (EEwsFolderPrivate, 1);
-	folder->priv = priv;
-
-	priv->error = NULL;
-	priv->folder_type = E_EWS_FOLDER_TYPE_UNKNOWN;
-	priv->foreign = FALSE;
+	folder->priv = G_TYPE_INSTANCE_GET_PRIVATE (folder, E_TYPE_EWS_FOLDER, EEwsFolderPrivate);
+	folder->priv->error = NULL;
+	folder->priv->folder_type = E_EWS_FOLDER_TYPE_UNKNOWN;
+	folder->priv->foreign = FALSE;
 }
 
 static gboolean
@@ -196,8 +188,10 @@ e_ews_folder_set_from_soap_parameter (EEwsFolder *folder,
 	}
 
 	subparam = e_soap_parameter_get_first_child_by_name (node, "DisplayName");
-	if (subparam)
+	if (subparam) {
 		priv->name = e_soap_parameter_get_string_value (subparam);
+		priv->escaped_name = e_ews_folder_utils_escape_name (priv->name);
+	}
 
 	subparam = e_soap_parameter_get_first_child_by_name (node, "UnreadCount");
 	if (subparam)
@@ -393,7 +387,18 @@ e_ews_folder_set_name (EEwsFolder *folder,
 	priv = folder->priv;
 
 	g_free (priv->name);
+	g_free (priv->escaped_name);
+
 	priv->name = g_strdup (new_name);
+	priv->escaped_name = e_ews_folder_utils_escape_name (priv->name);
+}
+
+const gchar *
+e_ews_folder_get_escaped_name (const EEwsFolder *folder)
+{
+	g_return_val_if_fail (E_IS_EWS_FOLDER (folder), NULL);
+
+	return folder->priv->escaped_name;
 }
 
 const EwsFolderId *
@@ -498,6 +503,70 @@ e_ews_folder_set_foreign (EEwsFolder *folder,
 	folder->priv->foreign = is_foreign;
 }
 
+/* escapes backslashes with \5C and forward slashes with \2F */
+gchar *
+e_ews_folder_utils_escape_name (const gchar *folder_name)
+{
+	gint ii, jj, count = 0;
+	gchar *res;
+
+	if (!folder_name)
+		return NULL;
+
+	for (ii = 0; folder_name[ii]; ii++) {
+		if (folder_name[ii] == '\\' || folder_name[ii] == '/')
+			count++;
+	}
+
+	if (!count)
+		return g_strdup (folder_name);
+
+	res = g_malloc0 (sizeof (gchar *) * (1 + ii + (2 * count)));
+	for (ii = 0, jj = 0; folder_name[ii]; ii++, jj++) {
+		if (folder_name[ii] == '\\') {
+			res[jj] = '\\';
+			res[jj + 1] = '5';
+			res[jj + 2] = 'C';
+			jj += 2;
+		} else if (folder_name[ii] == '/') {
+			res[jj] = '\\';
+			res[jj + 1] = '2';
+			res[jj + 2] = 'F';
+			jj += 2;
+		} else {
+			res[jj] = folder_name[ii];
+		}
+	}
+
+	res[jj] = '\0';
+
+	return res;
+}
+
+/* reverses e_ews_folder_utils_escape_name() processing */
+gchar *
+e_ews_folder_utils_unescape_name (const gchar *escaped_folder_name)
+{
+	gchar *res = g_strdup (escaped_folder_name);
+	gint ii, jj;
+
+	if (!res)
+		return res;
+
+	for (ii = 0, jj = 0; res[ii]; ii++, jj++) {
+		if (res[ii] == '\\' && g_ascii_isxdigit (res[ii + 1]) && g_ascii_isxdigit (res[ii + 2])) {
+			res[jj] = ((g_ascii_xdigit_value (res[ii + 1]) & 0xF) << 4) | (g_ascii_xdigit_value (res[ii + 2]) & 0xF);
+			ii += 2;
+		} else if (ii != jj) {
+			res[jj] = res[ii];
+		}
+	}
+
+	res[jj] = '\0';
+
+	return res;
+}
+
 gchar *
 e_ews_folder_utils_pick_color_spec (gint move_by,
                                     gboolean around_middle)
